@@ -334,7 +334,7 @@ impl<I: ConnId> QuicP2p<I> {
 // Private helpers
 
 // Bind a new socket with a local address
-fn bind(
+pub(crate) fn bind(
     endpoint_cfg: quinn::ServerConfig,
     local_addr: SocketAddr,
     allow_random_port: bool,
