@@ -9,53 +9,69 @@
 
 use std::{collections::BTreeMap, net::SocketAddr, sync::Arc};
 
+use tiny_keccak::{Hasher, Sha3};
 use tokio::sync::RwLock;
+use xor_name::XorName;
 
 // Pool for keeping open connections. Pooled connections are associated with a `ConnectionRemover`
 // which can be used to remove them from the pool.
 #[derive(Clone)]
-pub(crate) struct ConnectionPool {
-    store: Arc<RwLock<Store>>,
+pub(crate) struct ConnectionPool<I: ConnId> {
+    store: Arc<RwLock<Store<I>>>,
 }
 
-impl ConnectionPool {
+impl<I: ConnId> ConnectionPool<I> {
     pub fn new() -> Self {
         Self {
             store: Arc::new(RwLock::new(Store::default())),
         }
     }
 
-    pub async fn insert(&self, addr: SocketAddr, conn: quinn::Connection) -> ConnectionRemover {
+    pub async fn insert(
+        &self,
+        id: I,
+        addr: SocketAddr,
+        conn: quinn::Connection,
+    ) -> ConnectionRemover<I> {
         let mut store = self.store.write().await;
 
         let key = Key {
             addr,
             id: store.id_gen.next(),
         };
-        let _ = store.map.insert(key, conn);
+        let _ = store.id_map.insert(id, (conn.clone(), key));
+        let _ = store.key_map.insert(key, (conn, id));
 
         ConnectionRemover {
             store: self.store.clone(),
+            id,
             key,
         }
     }
 
-    pub async fn has(&self, addr: &SocketAddr) -> bool {
+    pub async fn has_addr(&self, addr: &SocketAddr) -> bool {
         let store = self.store.read().await;
 
         // Efficiently fetch the first entry whose key is equal to `key` and check if it exists
         store
-            .map
+            .key_map
             .range(Key::min(*addr)..=Key::max(*addr))
             .next()
             .is_some()
     }
 
+    #[allow(unused)]
+    pub async fn has_id(&self, id: &I) -> bool {
+        let store = self.store.read().await;
+
+        store.id_map.contains_key(id)
+    }
+
     pub async fn remove(&self, addr: &SocketAddr) -> Vec<quinn::Connection> {
         let mut store = self.store.write().await;
 
         let keys_to_remove = store
-            .map
+            .key_map
             .range_mut(Key::min(*addr)..=Key::max(*addr))
             .into_iter()
             .map(|(key, _)| *key)
@@ -63,20 +79,41 @@ impl ConnectionPool {
 
         keys_to_remove
             .iter()
-            .filter_map(|key| store.map.remove(&key))
+            .filter_map(|key| store.key_map.remove(key).map(|entry| entry.0))
             .collect::<Vec<_>>()
     }
 
-    pub async fn get(&self, addr: &SocketAddr) -> Option<(quinn::Connection, ConnectionRemover)> {
+    pub async fn get_by_id(&self, addr: &I) -> Option<(quinn::Connection, ConnectionRemover<I>)> {
         let store = self.store.read().await;
 
-        // Efficiently fetch the first entry whose key is equal to `key`.
-        let (key, conn) = store.map.range(Key::min(*addr)..=Key::max(*addr)).next()?;
+        let (conn, key) = store.id_map.get(addr)?;
 
-        let conn = conn.clone();
         let remover = ConnectionRemover {
             store: self.store.clone(),
             key: *key,
+            id: *addr,
+        };
+
+        Some((conn.clone(), remover))
+    }
+
+    pub async fn get_by_addr(
+        &self,
+        addr: &SocketAddr,
+    ) -> Option<(quinn::Connection, ConnectionRemover<I>)> {
+        let store = self.store.read().await;
+
+        // Efficiently fetch the first entry whose key is equal to `key`.
+        let (key, entry) = store
+            .key_map
+            .range(Key::min(*addr)..=Key::max(*addr))
+            .next()?;
+
+        let conn = entry.clone().0;
+        let remover = ConnectionRemover {
+            store: self.store.clone(),
+            key: *key,
+            id: entry.1,
         };
 
         Some((conn, remover))
@@ -85,29 +122,56 @@ impl ConnectionPool {
 
 // Handle for removing a connection from the pool.
 #[derive(Clone)]
-pub(crate) struct ConnectionRemover {
-    store: Arc<RwLock<Store>>,
+pub(crate) struct ConnectionRemover<I: ConnId> {
+    store: Arc<RwLock<Store<I>>>,
     key: Key,
+    id: I,
 }
 
-impl ConnectionRemover {
+impl<I: ConnId> ConnectionRemover<I> {
     // Remove the connection from the pool.
     pub async fn remove(&self) {
         let mut store = self.store.write().await;
-        let _ = store.map.remove(&self.key);
+        let _ = store.key_map.remove(&self.key);
+        let _ = store.id_map.remove(&self.id);
     }
 
     pub fn remote_addr(&self) -> &SocketAddr {
         &self.key.addr
     }
+
+    pub fn id(&self) -> I {
+        self.id
+    }
 }
 
 #[derive(Default)]
-struct Store {
-    map: BTreeMap<Key, quinn::Connection>,
+struct Store<I: ConnId> {
+    id_map: BTreeMap<I, (quinn::Connection, Key)>,
+    key_map: BTreeMap<Key, (quinn::Connection, I)>,
     id_gen: IdGen,
 }
 
+/// Unique key identifying a connection. Two connections will always have distict keys even if they
+/// have the same socket address.
+pub trait ConnId:
+    Clone + Copy + Eq + PartialEq + Ord + PartialOrd + Default + Send + Sync + 'static
+{
+    /// Generate
+    fn generate(socket_addr: &SocketAddr) -> Result<Self, Box<dyn std::error::Error>>;
+}
+
+impl ConnId for XorName {
+    fn generate(addr: &SocketAddr) -> Result<Self, Box<dyn std::error::Error>> {
+        let data = bincode::serialize(addr)?;
+        let mut hasher = Sha3::v256();
+        let mut output = [0u8; 32];
+        hasher.update(&data);
+        hasher.finalize(&mut output);
+        Ok(XorName(output))
+    }
+}
+
 // Unique key identifying a connection. Two connections will always have distict keys even if they
 // have the same socket address.
 #[derive(Clone, Copy, Eq, PartialEq, Ord, PartialOrd)]
